import axios from "axios";
import set from "lodash/set";
import get from "lodash/get";
import config from "../auth_config.json";
import keys from "../api_keys.json";

const instances = Object.entries(keys).reduce(
  (acc, [key, api]) => ({
    ...acc,
    [key]: {
      apiInstance: axios.create({
        baseURL: api.apiBase,
        headers: {
          "content-type": "application/json",
          "x-apikey": api.key,
          "cache-control": "no-cache"
        }
      })
    }
  }),
  {}
);

Object.keys(instances).forEach((instanceKey) => {
  instances[instanceKey].apiInstance.interceptors.response.use((response) => response.data || response);
  instances[instanceKey].get = (path) => instances[instanceKey].apiInstance.get(path);
  instances[instanceKey].post = (path, payload) => instances[instanceKey].apiInstance.post(path, payload);
});

const apiHeaders = {
  "content-type": "application/json",
  "x-apikey": config.apiKey,
  "cache-control": "no-cache"
};

const apiInstance = axios.create({
  baseURL: config.apiBase,
  headers: apiHeaders
});

apiInstance.interceptors.response.use((response) => response.data || response);

const updateData = (path, payload, method) => {
  console.log("path", path);
  if (!path) {
    throw new Error("Error, path must be specified when calling updateData endpoint");
  }
  if (path.split(".").length < 2) {
    throw new Error(`Error, path must have proper namespace in format "team.dataset".`);
  }
  if (!method || !["replace", "add"].includes(method)) {
    throw new Error(`Update method is not specified. It muset be one of ["replace", "add"]`);
  }
  return apiInstance.get("/rest/customdata").then((data) => {
    const [namespace, ...paths] = path.split(".");
    const destination = paths.join("");
    const index = data.findIndex(({ team }) => team === namespace);
    if (index === -1) {
      throw new Error(`${target} is not valid namepsace. Available namespaces ${data.map(({ team }) => team)}`);
    }
    let target = `[${index}].data.${destination}`;
    let newData;
    if (method === "replace") {
      newData = set(data, target, payload);
    } else {
      newData = set(data, `${target}[${[...(get(data, target) || [])].length}]`, payload);
    }
    return apiInstance.post("/rest/customdata", newData);
  });
};

const Api = {
  ...instances,
  updateData,
  get: (path) => apiInstance.get(path),
  post: (path, payload) => apiInstance.post(path, payload)
};

export default Api;
